@@ -3,9 +3,9 @@
 ## Author: Brice Ozenne
 ## Created: mar  5 2021 (12:59) 
 ## Version: 
-## Last-Updated: okt 18 2021 (11:19) 
+## Last-Updated: nov 13 2021 (16:35) 
 ##           By: Brice Ozenne
-##     Update #: 456
+##     Update #: 464
 ##----------------------------------------------------------------------
 ## 
 ### Commentary: 
@@ -90,9 +90,8 @@ score.lmm <- function(x, effects = "mean", data = NULL, p = NULL, indiv = FALSE,
                                         structure = x$design$vcov,
                                         data = data,
                                         var.outcome = x$outcome$var,
-                                        var.strata = x$strata$var, U.strata = x$strata$levels,
-                                        var.time = x$time$var, U.time = x$time$levels,
-                                        var.cluster = x$cluster$var,
+                                        U.strata = x$strata$levels, U.time = x$time$levels,
+                                        stratify.mean = x$opt$name=="gls",
                                         precompute.moments = test.precompute)
         }else{
             design <- x$design
@@ -127,6 +126,16 @@ score.lmm <- function(x, effects = "mean", data = NULL, p = NULL, indiv = FALSE,
         }
     }
 
+    ## re-order columns when converting to sd with strata (avoid sd0:0 sd0:1 sd1:0 sd1:1 sd2:0 sd2:1 ...)
+    if("variance" %in% effects && transform.k %in% c("sd","var","logsd","logvar") && x$strata$n>1 && transform.names){
+        out.name <- names(stats::coef(x, effects = effects, transform.sigma = transform.sigma, transform.k = transform.k, transform.rho = transform.rho, transform.names = TRUE))
+        if(indiv){
+            out <- out[,out.name,drop=FALSE]
+        }else{
+            out <- out[out.name]
+        }
+    }
+
     ## ** export
     return(out)
 }
